@@ -16,7 +16,7 @@ Aeut_comparison <- structure(list(Pop = structure(1:3,
     G = c(42.1928251121076, 28.7234042553191, 68.9723865877712), 
     lambda = c(0.976299287915825, 0.965185185185185, 0.985501444136235), 
     E.5 = c(0.721008688944842, 0.725926650260449, 0.720112175857993), 
-    Hexp = c(0.408951651286696, 0.33656220322887, 0.726202391505946),
+    Hexp = c(0.169711892488954, 0.158016764758338, 0.365053352719387), #c(0.408951651286696, 0.33656220322887, 0.726202391505946),
     Ia = c(2.90602921191748, 13.3024309367662, 14.3707995986407), 
     rbarD = c(0.0723700801886747, 0.281642324983496, 0.270617053778004), 
     File = structure(c(1L, 1L, 1L), class = "factor", .Label = "rootrot.csv")), 
@@ -35,7 +35,8 @@ pc_comparison <- structure(list(Pop = structure(1:5, .Label = c("1", "2", "3",
     lambda = c(0.887573964497041, 0.911242603550296, 0.902777777777778, 
              0.861111111111111, 0.944), 
     E.5 = c(0.935312405238733, 0.960842907662783, 0.958200460752105, 0.870390481833875, 0.819311895784525), 
-    Hexp = c(0.849679487179487, 0.83629191321499, 0.714756944444444, 0.750347222222222, 0.78384),
+    Hexp = c(0.617846153846154, 0.614461538461539, 0.485144927536232, 0.505072463768116, 
+             0.559212121212121), #c(0.849679487179487, 0.83629191321499, 0.714756944444444, 0.750347222222222, 0.78384),
     Ia = c(2.1580763424628, 1.87492360969648, 1.15572679509632, 1.157153633392, 1.93513179817012),
     rbarD = c(0.243225877705591, 0.212786561587854, 0.132460530412697, 0.13328661732193, 0.217470007471919),
     File = structure(c(1L, 1L, 1L, 1L, 1L), class = "factor", .Label = "partial_clone.dat")), 
